@@ -1,14 +1,18 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2018-2021, earthobservations developers.
 # Distributed under the MIT License. See LICENSE for more info.
+import datetime
 import logging
 import operator
-from abc import abstractmethod
+from abc import ABCMeta, abstractmethod
+from datetime import timedelta
 from enum import Enum
-from typing import Dict, Generator, List, Tuple, Union
+from typing import Dict, Generator, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
+import pytz
+from pandas.api.types import is_datetime64_any_dtype as is_datetime
 from pint import Quantity
 from pytz import timezone
 from tqdm import tqdm
@@ -24,8 +28,8 @@ from wetterdienst.util.logging import TqdmToLogger
 log = logging.getLogger(__name__)
 
 
-class ScalarValuesCore:
-    """ Core for sources of point data where data is related to a station """
+class ScalarValuesCore(metaclass=ABCMeta):
+    """Core for sources of point data where data is related to a station"""
 
     # Fields for type coercion, needed for separation from fields with actual data
     # that have to be parsed differently when having data in tabular form
@@ -38,12 +42,13 @@ class ScalarValuesCore:
         :return: list of strings representing the metadata fields/columns
         """
         if not self.stations.stations.tidy:
-            fields = [
+            return [
                 Columns.STATION_ID.value,
+                Columns.DATASET.value,
                 Columns.DATE.value,
             ]
         else:
-            fields = [
+            return [
                 Columns.STATION_ID.value,
                 Columns.DATASET.value,
                 Columns.PARAMETER.value,
@@ -52,28 +57,27 @@ class ScalarValuesCore:
                 Columns.QUALITY.value,
             ]
 
-        return fields
-
     # Fields for date coercion
     _date_fields = [Columns.DATE.value, Columns.FROM_DATE.value, Columns.TO_DATE.value]
 
-    # TODO: add data type (forecast, observation, ...)
-    # @property
-    # @abstractmethod
-    # def _has_quality(self) -> bool:
-    #     """Attribute that tells if a weather service has quality, which otherwise will
-    #     have to be set to NaN"""
-    #     pass
+    # TODO: add data type (mosmix, observation, ...)
 
     @property
     def data_tz(self) -> timezone:
-        """ Timezone of the published data """
+        """Timezone of the published data"""
         return timezone(self._data_tz.value)
 
     @property
     @abstractmethod
     def _data_tz(self) -> Timezone:
-        """ Timezone enumeration of published data. """
+        """Timezone enumeration of published data."""
+        pass
+
+    @property
+    @abstractmethod
+    def _date_parameters(self) -> Tuple[str]:
+        """Declaration of irregular parameters which will have to be parsed differently
+        then others e.g. when a parameter is a date."""
         pass
 
     @property
@@ -86,17 +90,16 @@ class ScalarValuesCore:
     @property
     @abstractmethod
     def _integer_parameters(self) -> Tuple[str]:
-        """ Integer parameters that will be parsed to integers. """
+        """Integer parameters that will be parsed to integers."""
         pass
 
     @property
     @abstractmethod
     def _string_parameters(self) -> Tuple[str]:
-        """ String parameters that will be parsed to integers. """
+        """String parameters that will be parsed to integers."""
         pass
 
-    @property
-    def _complete_dates(self) -> pd.DatetimeIndex:
+    def _get_complete_dates(self, station_id) -> pd.DatetimeIndex:
         """
         Complete datetime index for the requested start and end date, used for
         building a complementary pandas DataFrame with the date column on which
@@ -111,24 +114,65 @@ class ScalarValuesCore:
         elif self.stations.stations.resolution == Resolution.ANNUAL:
             end_date += pd.Timedelta(year=366)
 
-        date_range = pd.date_range(
-            start_date,
-            end_date,
+        if self._data_tz == Timezone.DYNAMIC:
+            timezone_ = self._get_timezone_from_station(station_id)
+        else:
+            timezone_ = self.data_tz
+
+        tz_delta = self._get_timedelta_from_timezones(timezone_, pytz.UTC)
+
+        # TODO: better manage start and end date and its timezone
+        return pd.date_range(
+            start_date + datetime.timedelta(hours=tz_delta),
+            end_date + datetime.timedelta(hours=tz_delta),
             freq=self.stations.frequency.value,
-            tz=self.data_tz,
+            tz=pytz.UTC,
         )
 
-        return date_range
+    def _get_timezone_from_station(self, station_id: str) -> timezone:
+        """
+        Get timezone information for explicit station that is used to set the
+        correct timezone for the timestamps of the returned values
 
-    @property
-    def _base_df(self) -> pd.DataFrame:
+        :param station_id: station id string
+        :return: timezone
+        """
+
+        stations = self.stations.df
+
+        station = stations[stations[Columns.STATION_ID.value] == station_id]
+
+        longitude, latitude = station.loc[:, [Columns.LONGITUDE.value, Columns.LATITUDE.value]].T.values
+
+        tz_string = self._tf.timezone_at(lng=longitude, lat=latitude)
+
+        return timezone(tz_string)
+
+    @staticmethod
+    def _get_timedelta_from_timezones(tz1: timezone, tz2: timezone) -> timedelta:
+        """Method to get timedelta in hours from two timezones
+
+        :param tz1:
+        :param tz2:
+        :return:
+        """
+        dt_1970 = datetime.datetime(1970, 1, 1)
+
+        dt_1970_tz1 = tz1.localize(dt_1970)
+        dt_1970_tz2 = tz2.localize(dt_1970).astimezone(tz1)
+
+        dt_diff = dt_1970_tz1 - dt_1970_tz2
+
+        return dt_diff.days * 24 + dt_diff.seconds / 3600
+
+    def _get_base_df(self, station_id: str) -> pd.DataFrame:
         """
         Base dataframe which is used for creating empty dataframes if no data is
         found or for merging other dataframes on the full dates
 
         :return: pandas DataFrame with a date column with complete dates
         """
-        return pd.DataFrame({Columns.DATE.value: self._complete_dates})
+        return pd.DataFrame({Columns.DATE.value: self._get_complete_dates(station_id)})
 
     def convert_values_to_si(self, df: pd.DataFrame, dataset) -> pd.DataFrame:
         """
@@ -138,15 +182,19 @@ class ScalarValuesCore:
         :param dataset: dataset for which the conversion factors are created
         :return: pandas DataFrame with converted (SI) values
         """
-
-        def _convert_values_to_si(series):
+        # TODO: THIS DOESNT CURRENTLY WORK WITH PREFIXED TIDY DATA
+        def _convert_values_to_si(series, parameter: Optional[str] = None) -> pd.Series:
             """
             Helper function to apply conversion factors column wise to a pandas DataFrame
 
             :param series: pandas Series that should be converted
+            :param parameter: optional parameter string used for tidy data
             :return: converted pandas Series
             """
-            op, factor = conversion_factors.get(series.name, (None, None))
+            if not parameter:
+                parameter = series.name
+
+            op, factor = conversion_factors.get(parameter, (None, None))
 
             if not op or not factor:
                 return series
@@ -155,13 +203,18 @@ class ScalarValuesCore:
 
         conversion_factors = self._create_conversion_factors(dataset)
 
-        df = df.apply(_convert_values_to_si, axis=0)
+        if self.stations.stations._has_tidy_data:
+            df_si = pd.DataFrame()
+            for par, group in df.groupby(Columns.PARAMETER.value):
+                group[Columns.VALUE.value] = _convert_values_to_si(group[Columns.VALUE.value], par)
 
-        return df
+                df_si = df_si.append(group)
 
-    def _create_conversion_factors(
-        self, dataset
-    ) -> Dict[str, Tuple[Union[operator.add, operator.mul], float]]:
+            return df_si
+
+        return df.apply(_convert_values_to_si, axis=0)
+
+    def _create_conversion_factors(self, dataset) -> Dict[str, Tuple[Union[operator.add, operator.mul], float]]:
         """
         Function to create conversion factors based on a given dataset
 
@@ -187,13 +240,9 @@ class ScalarValuesCore:
             parameter = parameter.name
 
             if self.stations.stations._unique_dataset:
-                parameter_value = self.stations.stations._dataset_tree[
-                    dataset_accessor
-                ][parameter].value
+                parameter_value = self.stations.stations._dataset_tree[dataset_accessor][parameter].value
             else:
-                parameter_value = self.stations.stations._dataset_tree[
-                    dataset_accessor
-                ][dataset][parameter].value
+                parameter_value = self.stations.stations._dataset_tree[dataset_accessor][dataset][parameter].value
 
             if si_unit == SIUnit.KILOGRAM_PER_SQUARE_METER.value:
                 # Fixed conversion factors to kg / m², as it only applies
@@ -201,9 +250,7 @@ class ScalarValuesCore:
                 if origin_unit == OriginUnit.MILLIMETER.value:
                     conversion_factors[parameter_value] = (operator.mul, 1)
                 else:
-                    raise ValueError(
-                        "manually set conversion factor for precipitation unit"
-                    )
+                    raise ValueError("manually set conversion factor for precipitation unit")
             elif si_unit == SIUnit.DEGREE_KELVIN.value:
                 # Apply offset addition to temperature measurements
                 # Take 0 as this is appropriate for adding on other numbers
@@ -233,28 +280,22 @@ class ScalarValuesCore:
         return cls(stations)
 
     def __eq__(self, other):
-        """ Equal method of request object """
+        """Equal method of request object"""
         return (
             self.stations.station_id == other.stations.station_id
             and self.stations.parameter == other.stations.parameter
             and self.stations.start_date == other.stations.start_date
             and self.stations.end_date == other.stations.end_date
         )
-        pass
 
     def __str__(self):
-        """ Str representation of request object """
+        """Str representation of request object"""
         # TODO: include source
         # TODO: include data type
-        station_ids_joined = "& ".join(
-            [str(station_id) for station_id in self.stations.station_id]
-        )
+        station_ids_joined = "& ".join([str(station_id) for station_id in self.stations.station_id])
 
         parameters_joined = "& ".join(
-            [
-                parameter.value
-                for parameter, parameter_set in self.stations.stations.parameter
-            ]
+            [parameter.value for parameter, parameter_set in self.stations.stations.parameter]
         )
 
         return ", ".join(
@@ -265,11 +306,8 @@ class ScalarValuesCore:
                 str(self.stations.end_date),
             ]
         )
-        pass
 
-    def _create_empty_station_parameter_df(
-        self, station_id: str, parameter: Enum, dataset: Enum
-    ) -> pd.DataFrame:
+    def _create_empty_station_parameter_df(self, station_id: str, parameter: Enum, dataset: Enum) -> pd.DataFrame:
         """
         Function to create an empty DataFrame
         :param station_id:
@@ -295,7 +333,7 @@ class ScalarValuesCore:
                 if par.name.startswith("QUALITY"):
                     continue
 
-                par_df = self._base_df
+                par_df = self._get_base_df(station_id)
                 par_df[Columns.PARAMETER.value] = par.value
 
                 data.append(par_df)
@@ -303,7 +341,7 @@ class ScalarValuesCore:
             df = pd.concat(data)
 
             df[Columns.STATION_ID.value] = station_id
-            df[Columns.DATASET.value] = dataset.name
+            df[Columns.DATASET.value] = dataset.name.lower()
             df[Columns.VALUE.value] = pd.NA
             df[Columns.QUALITY.value] = pd.NA
 
@@ -317,45 +355,42 @@ class ScalarValuesCore:
             if self.stations.stations.start_date:
                 return pd.DataFrame(None, columns=columns)
 
-            df = self._base_df
+            df = self._get_base_df(station_id)
 
             df = df.reindex(columns=columns)
 
             df[Columns.STATION_ID.value] = station_id
+            df[Columns.DATASET.value] = dataset.name.lower()
 
             return df
 
-    def _build_complete_df(
-        self, df: pd.DataFrame, station_id: str, parameter: Enum, dataset: Enum
-    ) -> pd.DataFrame:
-        # For cases where requests are not defined by start and end date but rather by
-        # periods, use the returned df without modifications
-        # We may put a standard date range here if no data is found
-        if not self.stations.start_date:
-            return df
+    def _build_complete_df(self, df: pd.DataFrame, station_id: str, parameter: Enum, dataset: Enum) -> pd.DataFrame:
+        """Method to build a complete df with all dates from start to end date included.
 
+        :param df:
+        :param station_id:
+        :param parameter:
+        :param dataset:
+        :return:
+        """
         if parameter != dataset or not self.stations.stations.tidy:
+            base_df = self._get_base_df(station_id)
+
             df = pd.merge(
-                left=self._base_df,
+                left=base_df,
                 right=df,
                 left_on=Columns.DATE.value,
                 right_on=Columns.DATE.value,
                 how="left",
             )
 
-            df[Columns.STATION_ID.value] = station_id
-
             if self.stations.tidy:
                 df[Columns.PARAMETER.value] = parameter.value
-                df[Columns.PARAMETER.value] = pd.Categorical(
-                    df[Columns.PARAMETER.value]
-                )
+                df[Columns.PARAMETER.value] = pd.Categorical(df[Columns.PARAMETER.value])
 
                 if dataset:
                     df[Columns.DATASET.value] = dataset.name.lower()
-                    df[Columns.DATASET.value] = pd.Categorical(
-                        df[Columns.DATASET.value]
-                    )
+                    df[Columns.DATASET.value] = pd.Categorical(df[Columns.DATASET.value])
 
             return df
         else:
@@ -364,28 +399,22 @@ class ScalarValuesCore:
                 if self.stations.stations._unique_dataset:
                     parameter_ = parse_enumeration_from_template(
                         parameter,
-                        self.stations.stations._parameter_base[
-                            self.stations.resolution.name
-                        ],
+                        self.stations.stations._parameter_base[self.stations.resolution.name],
                     )
                 else:
                     parameter_ = parse_enumeration_from_template(
                         parameter,
-                        self.stations.stations._dataset_tree[
-                            self.stations.resolution.name
-                        ][dataset.name],
+                        self.stations.stations._dataset_tree[self.stations.resolution.name][dataset.name],
                     )
 
                 df = pd.merge(
-                    left=self._base_df,
+                    left=self._get_base_df(station_id),
                     right=group,
                     left_on=Columns.DATE.value,
                     right_on=Columns.DATE.value,
                     how="left",
                 )
 
-                df[Columns.STATION_ID.value] = station_id
-
                 df[Columns.PARAMETER.value] = parameter_.value
 
                 df[Columns.DATASET.value] = dataset.name.lower()
@@ -406,9 +435,7 @@ class ScalarValuesCore:
 
         columns.extend(df.columns.difference(columns, sort=False))
 
-        df = df.reindex(columns=columns)
-
-        return df
+        return df.reindex(columns=columns)
 
     def query(self) -> Generator[ValuesResult, None, None]:
         """
@@ -418,51 +445,77 @@ class ScalarValuesCore:
 
         :return:
         """
+
         for station_id in self.stations.station_id:
             # TODO: add method to return empty result with correct response string e.g.
             #  station id not available
             station_data = []
 
             for parameter, dataset in self.stations.parameter:
-                parameter_df = self._collect_station_parameter(
-                    station_id, parameter, dataset
-                )
+                # TODO: For now skip date based parameters as we didn't
+                #  yet decide how to convert those values to floats to
+                #  make them conform with tidy shape
+                single_tidy_date_parameters = self.stations.stations.tidy and parameter.value in self._date_parameters
+                if single_tidy_date_parameters:
+                    log.warning(
+                        f"parameter {parameter.value} is skipped in tidy format "
+                        f"as the date parameter is currently not converted to float"
+                    )
+                    continue
+
+                parameter_df = self._collect_station_parameter(station_id, parameter, dataset)
 
                 if parameter_df.empty:
                     continue
 
-                # Merge on full date range if values are found to ensure result
-                # even if no actual values exist
-                self._coerce_date_fields(parameter_df)
+                self._coerce_date_fields(parameter_df, station_id)
 
+                # TODO: we are coercing values here for conversion of units
+                #  however we later again coerce when concatenating DataFrames
                 parameter_df = self._coerce_parameter_types(parameter_df)
 
                 if self.stations.stations.si_units:
                     parameter_df = self.convert_values_to_si(parameter_df, dataset)
 
                 if self.stations.stations.tidy:
-                    parameter_df = self.tidy_up_df(parameter_df, dataset)
+                    if not self.stations.stations._has_tidy_data:
+                        parameter_df = self.tidy_up_df(parameter_df, dataset)
 
                     if parameter != dataset:
-                        parameter_df = parameter_df[
-                            parameter_df[Columns.PARAMETER.value]
-                            == parameter.value.lower()
-                        ]
+                        parameter_df = parameter_df[parameter_df[Columns.PARAMETER.value] == parameter.value.lower()]
+                elif self.stations.stations._has_tidy_data:
+                    parameter_df = self.tabulate_df(parameter_df)
 
-                parameter_df = self._build_complete_df(
-                    parameter_df, station_id, parameter, dataset
-                )
+                # Skip date fields in tidy format, no further check required as still
+                # "normal" parameters should be available
+                if self.stations.stations.tidy and self._date_parameters:
+                    parameter_df = parameter_df[~parameter_df[Columns.PARAMETER.value].isin(self._date_parameters)]
+                    log.warning(
+                        f"parameters {self._date_parameters} are skipped in tidy format "
+                        f"as the date parameters are currently not converted to floats"
+                    )
+                    if parameter_df.empty:
+                        continue
 
-                parameter_df = self._organize_df_columns(parameter_df)
+                # Merge on full date range if values are found to ensure result
+                # even if no actual values exist
+                # For cases where requests are not defined by start and end date but rather by
+                # periods, use the returned df without modifications
+                # We may put a standard date range here if no data is found
+                if self.stations.stations.start_date:
+                    parameter_df = self._build_complete_df(parameter_df, station_id, parameter, dataset)
 
                 station_data.append(parameter_df)
 
             try:
                 station_df = pd.concat(station_data, ignore_index=True)
             except ValueError:
-                station_df = self._create_empty_station_parameter_df(
-                    station_id, parameter
-                )
+                station_df = self._create_empty_station_parameter_df(station_id, parameter, dataset)
+
+            station_df = self._organize_df_columns(station_df)
+
+            station_df[Columns.STATION_ID.value] = station_id
+            station_df[Columns.DATASET.value] = dataset.name.lower()
 
             station_df = self._coerce_meta_fields(station_df)
 
@@ -487,9 +540,7 @@ class ScalarValuesCore:
             yield ValuesResult(stations=self.stations, df=station_df)
 
     @abstractmethod
-    def _collect_station_parameter(
-        self, station_id: str, parameter: Enum, dataset: Enum
-    ) -> pd.DataFrame:
+    def _collect_station_parameter(self, station_id: str, parameter: Enum, dataset: Enum) -> pd.DataFrame:
         """
         Implementation of data collection for a station id plus parameter from the
         specified weather service. Takes care of the gathering of the data and putting
@@ -513,47 +564,85 @@ class ScalarValuesCore:
         """
         df = self._tidy_up_df(df, dataset)
 
-        df[Columns.DATASET.value] = pd.Series(
-            dataset.name.lower(), index=df.index, dtype=str
-        )
-
+        df[Columns.DATASET.value] = dataset.name.lower()
         df[Columns.VALUE.value] = pd.to_numeric(df[Columns.VALUE.value]).astype(float)
 
         if Columns.QUALITY.value not in df:
             df[Columns.QUALITY.value] = np.nan
 
-        df[Columns.QUALITY.value] = pd.to_numeric(df[Columns.QUALITY.value]).astype(
-            float
-        )
+        df[Columns.QUALITY.value] = pd.to_numeric(df[Columns.QUALITY.value]).astype(float)
 
+        # Set quality of NaN values to NaN as well
         df.loc[df[Columns.VALUE.value].isna(), Columns.QUALITY.value] = np.NaN
 
         return df
 
-    @abstractmethod
     def _tidy_up_df(self, df: pd.DataFrame, dataset) -> pd.DataFrame:
         """
-        Abstract method to be implemented by services to tidy a DataFrame
+        Abstract method to be implemented by services to tidy a DataFrame if
+        tidy is not provided tidy by default
 
         :param df:
         :return:
         """
-        pass
+        if not self.stations.stations._has_tidy_data:
+            raise NotImplementedError("implement _tidy_up_df method to tidy data")
 
-    def _coerce_date_fields(self, df: pd.DataFrame) -> pd.DataFrame:
+    @staticmethod
+    def tabulate_df(df: pd.DataFrame) -> pd.DataFrame:
+        """
+        Method to tabulate a dataframe with each row having one timestamp and
+        all parameter values and corresponding quality levels.
+
+        Example:
+
+        date         parameter                  value   quality
+        1971-01-01   precipitation_height       0       0
+        1971-01-01   temperature_air_mean_200   10      0
+
+        becomes
+
+        date         precipitation_height   qn_precipitation_height
+        1971-01-01   0                      0
+            temperature_air_mean_200    ...
+            10                          ...
+
+        :param df: pandas.DataFrame with tidy data
+        :returns pandas.DataFrame with tabulated data e.g. pairwise columns of values
+        and quality flags
+        """
+        df_tabulated = pd.DataFrame({Columns.DATE.value: df[Columns.DATE.value]})
+
+        for parameter, parameter_df in df.groupby(by=[df[Columns.PARAMETER.value]]):
+            # Build quality columm name
+            parameter_quality = f"{Columns.QUALITY_PREFIX.value}_{parameter}"
+
+            # Add values
+            df_tabulated[parameter] = parameter_df[Columns.VALUE.value]
+            # Add quality levels
+            df_tabulated[parameter_quality] = parameter_df[Columns.QUALITY.value]
+
+        return df_tabulated
+
+    def _coerce_date_fields(self, df: pd.DataFrame, station_id: str) -> pd.DataFrame:
         """
         Function for coercion of possible date fields
 
         :param df:
         :return:
         """
+        if self._data_tz == Timezone.DYNAMIC:
+            timezone_ = self._get_timezone_from_station(station_id)
+        else:
+            timezone_ = self.data_tz
+
         for column in (
             Columns.DATE.value,
             Columns.FROM_DATE.value,
             Columns.TO_DATE.value,
         ):
             try:
-                df[column] = self._coerce_dates(df[column])
+                df[column] = self._coerce_dates(df[column], timezone_)
             except KeyError:
                 pass
 
@@ -570,21 +659,19 @@ class ScalarValuesCore:
         :param df: pandas.DataFrame with the "fresh" data
         :return: pandas.DataFrame with meta fields being coerced
         """
-        df[Columns.STATION_ID.value] = self._parse_station_id(
-            df[Columns.STATION_ID.value]
-        ).astype("category")
+        df[Columns.STATION_ID.value] = self._parse_station_id(df[Columns.STATION_ID.value]).astype("category")
+
+        # TODO: why do we need this (again)?
+        df[Columns.DATE.value] = pd.to_datetime(df[Columns.DATE.value], infer_datetime_format=True)
+
+        df[Columns.DATASET.value] = self._coerce_strings(df[Columns.DATASET.value]).astype("category")
 
         if self.stations.stations.tidy:
-            for column in (Columns.DATASET.value, Columns.PARAMETER.value):
-                df[column] = self._coerce_strings(df[column]).astype("category")
+            df[Columns.PARAMETER.value] = self._coerce_strings(df[Columns.PARAMETER.value]).astype("category")
+            df[Columns.VALUE.value] = pd.to_numeric(df[Columns.VALUE.value]).astype(float)
 
-            df[Columns.VALUE.value] = pd.to_numeric(df[Columns.VALUE.value]).astype(
-                float
-            )
-
-            df[Columns.QUALITY.value] = pd.to_numeric(df[Columns.QUALITY.value]).astype(
-                float
-            )
+            # TODO: may coerce more carefully quality codes or replace them by numbers
+            df[Columns.QUALITY.value] = pd.to_numeric(df[Columns.QUALITY.value], errors="coerce").astype(float)
 
         return df
 
@@ -598,7 +685,7 @@ class ScalarValuesCore:
         """
         return self.stations.stations._parse_station_id(series)
 
-    def _coerce_dates(self, series: pd.Series) -> pd.Series:
+    def _coerce_dates(self, series: pd.Series, timezone_: timezone) -> pd.Series:
         """
         Method to parse dates in the pandas.DataFrame. Leverages the data timezone
         attribute to ensure correct comparison of dates.
@@ -606,9 +693,14 @@ class ScalarValuesCore:
         :param series:
         :return:
         """
-        return pd.to_datetime(series, infer_datetime_format=True).dt.tz_localize(
-            self.data_tz
-        )
+        if not is_datetime(series):
+            series = pd.to_datetime(series, infer_datetime_format=True)
+
+        try:
+            return series.dt.tz_localize(timezone_)
+        except TypeError:
+            # Assuming the dates already have a timezone we will convert dates to our required timezone
+            return series.dt.tz_convert(timezone_)
 
     @staticmethod
     def _coerce_integers(series: pd.Series) -> pd.Series:
@@ -619,11 +711,7 @@ class ScalarValuesCore:
         :param series:
         :return:
         """
-        return (
-            pd.to_numeric(series, errors="coerce")
-            .astype(pd.Float64Dtype())
-            .astype(pd.Int64Dtype())
-        )
+        return pd.to_numeric(series, errors="coerce").astype(pd.Float64Dtype()).astype(pd.Int64Dtype())
 
     @staticmethod
     def _coerce_strings(series: pd.Series) -> pd.Series:
@@ -655,10 +743,7 @@ class ScalarValuesCore:
         :return:
         """
         if self._irregular_parameters:
-            raise NotImplementedError(
-                "implement _parse_irregular_parameter "
-                "method to parse irregular parameters"
-            )
+            raise NotImplementedError("implement _parse_irregular_parameter " "method to parse irregular parameters")
 
         return pd.Series(series)
 
@@ -674,9 +759,7 @@ class ScalarValuesCore:
                 continue
             if column in self._irregular_parameters:
                 df[column] = self._coerce_irregular_parameter(df[column])
-            elif column in self._integer_parameters or column.startswith(
-                Columns.QUALITY_PREFIX.value
-            ):
+            elif column in self._integer_parameters or column.startswith(Columns.QUALITY_PREFIX.value):
                 df[column] = self._coerce_integers(df[column])
             elif column in self._string_parameters:
                 df[column] = self._coerce_strings(df[column])
@@ -695,15 +778,14 @@ class ScalarValuesCore:
 
         tqdm_out = TqdmToLogger(log, level=logging.INFO)
 
-        for result in tqdm(
-            self.query(), total=len(self.stations.station_id), file=tqdm_out
-        ):
+        for result in tqdm(self.query(), total=len(self.stations.station_id), file=tqdm_out):
             data.append(result.df)
 
-        if not data:
-            raise ValueError("No data available for given constraints")
-
-        df = pd.concat(data, ignore_index=True)
+        try:
+            df = pd.concat(data, ignore_index=True)
+        except ValueError:
+            log.error("No data available for given constraints")
+            return ValuesResult(stations=self.stations, df=pd.DataFrame())
 
         # Have to reapply category dtype after concatenation
         df = self._coerce_meta_fields(df)
@@ -722,9 +804,7 @@ class ScalarValuesCore:
         if not self.stations.tidy:
             df = df.rename(columns=hcnm)
         else:
-            df[Columns.PARAMETER.value] = df[
-                Columns.PARAMETER.value
-            ].cat.rename_categories(hcnm)
+            df[Columns.PARAMETER.value] = df[Columns.PARAMETER.value].cat.rename_categories(hcnm)
 
         return df
 
@@ -735,9 +815,8 @@ class ScalarValuesCore:
 
         :return:
         """
-        hcnm = {
-            parameter.value: parameter.name.lower()
-            for parameter in self.stations.stations._parameter_base
-        }
 
-        return hcnm
+        return {
+            parameter.value: parameter.name.lower()
+            for parameter in self.stations.stations._parameter_base[self.stations.stations.resolution.name]
+        }
