@@ -10,50 +10,48 @@ from typing import Generator, Optional, Tuple, Union
 
 import pandas as pd
 import requests
+from fsspec.implementations.cached import WholeFileCacheFileSystem
+from fsspec.implementations.http import HTTPFileSystem
+from pandas._libs.tslibs.offsets import YearEnd
 from requests.adapters import HTTPAdapter
-from requests_ftp.ftp import FTPSession
 from urllib3 import Retry
 
 from wetterdienst.core.scalar.request import ScalarRequestCore
 from wetterdienst.core.scalar.values import ScalarValuesCore
 from wetterdienst.exceptions import FailedDownload
 from wetterdienst.metadata.columns import Columns
+from wetterdienst.metadata.datarange import DataRange
+from wetterdienst.metadata.kind import Kind
 from wetterdienst.metadata.period import Period, PeriodType
 from wetterdienst.metadata.provider import Provider
 from wetterdienst.metadata.resolution import Resolution, ResolutionType
 from wetterdienst.metadata.timezone import Timezone
 from wetterdienst.provider.eccc.observation.metadata.dataset import (
     EcccObservationDataset,
-    EcccObservationDatasetTree,
-)
-from wetterdienst.provider.eccc.observation.metadata.parameter import (
     EcccObservationParameter,
 )
 from wetterdienst.provider.eccc.observation.metadata.resolution import (
     EccObservationResolution,
 )
-from wetterdienst.provider.eccc.observation.metadata.unit import (
-    EcccObservationUnitOrigin,
-    EcccObservationUnitSI,
-)
-from wetterdienst.util.cache import payload_cache_twelve_hours
+from wetterdienst.provider.eccc.observation.metadata.unit import EcccObservationUnit
+from wetterdienst.util.cache import CacheExpiry, cache_dir
 
 log = logging.getLogger(__name__)
 
 
 class EcccObservationValues(ScalarValuesCore):
-    _string_parameters = []
-    _integer_parameters = []
-    _irregular_parameters = []
+
+    _string_parameters = ()
+    _integer_parameters = ()
+    _irregular_parameters = ()
+    _date_parameters = ()
 
     _data_tz = Timezone.UTC
 
     _has_quality = True
 
     _session = requests.Session()
-    _session.mount(
-        "https://", HTTPAdapter(max_retries=Retry(total=10, connect=5, read=5))
-    )
+    _session.mount("https://", HTTPAdapter(max_retries=Retry(total=10, connect=5, read=5)))
 
     _base_url = (
         "https://climate.weather.gc.ca/climate_data/bulk_data_e.html?"
@@ -70,7 +68,7 @@ class EcccObservationValues(ScalarValuesCore):
 
     @property
     def _timeframe(self) -> str:
-        """ internal timeframe string for resolution """
+        """internal timeframe string for resolution"""
         return self._timeframe_mapping.get(self.stations.stations.resolution)
 
     _time_step_mapping = {
@@ -82,29 +80,25 @@ class EcccObservationValues(ScalarValuesCore):
 
     @property
     def _time_step(self):
-        """ internal time step string for resolution """
+        """internal time step string for resolution"""
         return self._time_step_mapping.get(self.stations.stations.resolution)
 
     def _create_humanized_parameters_mapping(self):
         # TODO: change to something general, depending on ._has_datasets
-        hcnm = {
+        return {
             parameter.value: parameter.name.lower()
-            for parameter in self.stations.stations._parameter_base[
-                self.stations.stations.resolution.name
-            ]
+            for parameter in self.stations.stations._parameter_base[self.stations.stations.resolution.name]
         }
 
-        return hcnm
-
     def _tidy_up_df(self, df: pd.DataFrame, dataset) -> pd.DataFrame:
         """
         Tidy up dataframe pairwise by column 'DATE', 'Temp (°C)', 'Temp Flag', ...
 
-        :param df:
-        :return:
+        :param df: DataFrame with loaded data
+        :return: tidied DataFrame
         """
 
-        df_tidy = pd.DataFrame()
+        data = []
 
         columns = df.columns
         for parameter_column, quality_column in zip(columns[1::2], columns[2::2]):
@@ -116,23 +110,25 @@ class EcccObservationValues(ScalarValuesCore):
                 }
             )
             df_parameter[Columns.PARAMETER.value] = parameter_column
-            df_tidy = df_tidy.append(df_parameter)
+            data.append(df_parameter)
 
-        return df_tidy
+        try:
+            return pd.concat(data, ignore_index=True)
+        except ValueError:
+            # TODO: add logging
+            return pd.DataFrame()
 
     def _collect_station_parameter(
-        self, station_id: str, parameter: Tuple[Enum, Enum]
+        self, station_id: str, parameter: EcccObservationParameter, dataset: Enum
     ) -> pd.DataFrame:
         """
 
-        :param station_id:
-        :param parameter:
-        :return:
+        :param station_id: station id being queried
+        :param parameter: parameter being queried
+        :param dataset: dataset of query, can be skipped as ECCC has unique dataset
+        :return: pandas.DataFrame with data
         """
-        # parameter, dataset = parameter
-        meta = self.stations.df[
-            self.stations.df[Columns.STATION_ID.value] == station_id
-        ]
+        meta = self.stations.df[self.stations.df[Columns.STATION_ID.value] == station_id]
 
         name, from_date, to_date = (
             meta[
@@ -154,9 +150,7 @@ class EcccObservationValues(ScalarValuesCore):
         start_date = self.stations.stations.start_date
         end_date = self.stations.stations.end_date
 
-        start_year = start_year and max(
-            start_year, start_date and start_date.year or start_year
-        )
+        start_year = start_year and max(start_year, start_date and start_date.year or start_year)
         end_year = end_year and min(end_year, end_date and end_date.year or end_year)
 
         # Following lines may partially be based on @Zeitsperre's canada-climate-python
@@ -207,9 +201,7 @@ class EcccObservationValues(ScalarValuesCore):
 
         return df
 
-    def _create_file_urls(
-        self, station_id: str, start_year: int, end_year: int
-    ) -> Generator[str, None, None]:
+    def _create_file_urls(self, station_id: str, start_year: int, end_year: int) -> Generator[str, None, None]:
         """
 
         :param station_id:
@@ -225,9 +217,7 @@ class EcccObservationValues(ScalarValuesCore):
 
         # For hourly data request only necessary data to reduce amount of data being
         # downloaded and parsed
-        for date in pd.date_range(
-            f"{start_year}-01-01", f"{end_year + 1}-01-01", freq=freq, closed=None
-        ):
+        for date in pd.date_range(f"{start_year}-01-01", f"{end_year + 1}-01-01", freq=freq, closed=None):
             url = self._base_url.format(int(station_id), self._timeframe)
 
             url += f"&Year={date.year}"
@@ -250,6 +240,7 @@ class EcccObservationRequest(ScalarRequestCore):
     """
 
     provider = Provider.ECCC
+    kind = Kind.OBSERVATION
 
     _tz = Timezone.UTC
 
@@ -258,14 +249,14 @@ class EcccObservationRequest(ScalarRequestCore):
     _period_type = PeriodType.FIXED
     _period_base = Period.HISTORICAL
     _parameter_base = EcccObservationParameter  # replace with parameter enumeration
-
+    _data_range = DataRange.LOOSELY
     _has_datasets = True
     _dataset_base = EcccObservationDataset
-    _dataset_tree = EcccObservationDatasetTree
+    _dataset_tree = EcccObservationParameter
     _unique_dataset = True
+    _has_tidy_data = False
 
-    _origin_unit_tree = EcccObservationUnitOrigin
-    _si_unit_tree = EcccObservationUnitSI
+    _unit_tree = EcccObservationUnit
 
     _values = EcccObservationValues
 
@@ -318,19 +309,20 @@ class EcccObservationRequest(ScalarRequestCore):
         resolution: Union[EccObservationResolution, Resolution],
         start_date: Optional[datetime] = None,
         end_date: Optional[datetime] = None,
-        humanize: bool = True,
-        tidy: bool = True,
-        si_units: bool = True,
     ):
+        """
+
+        :param parameter: parameter or list of parameters that are being queried
+        :param resolution: resolution of data
+        :param start_date: start date for values filtering
+        :param end_date: end date for values filtering
+        """
         super(EcccObservationRequest, self).__init__(
             parameter=parameter,
             resolution=resolution,
             period=Period.HISTORICAL,
             start_date=start_date,
             end_date=end_date,
-            humanize=humanize,
-            tidy=tidy,
-            si_units=si_units,
         )
 
     def _all(self) -> pd.DataFrame:
@@ -346,43 +338,42 @@ class EcccObservationRequest(ScalarRequestCore):
 
         df = df.rename(columns=self._columns_mapping)
 
+        df[Columns.TO_DATE.value] = pd.to_datetime(df[Columns.TO_DATE.value]) + YearEnd()
+
         return df
 
     @staticmethod
-    @payload_cache_twelve_hours.cache_on_arguments()
-    def _download_stations() -> bytes:
+    def _download_stations() -> BytesIO:
         """
         Download station list from ECCC FTP server.
 
         :return: CSV payload
         """
 
-        ftp_url = (
-            "ftp://client_climate:foobar@ftp.tor.ec.gc.ca"
-            "/Pub/Get_More_Data_Plus_de_donnees/Station Inventory EN.csv"
-        )
+        gdrive_url = "https://drive.google.com/uc?id=1HDRnj41YBWpMioLPwAFiLlK4SK8NV72C"
 
         http_url = (
-            "https://raw.githubusercontent.com/earthobservations/testdata"
-            "/main/ftp.tor.ec.gc.ca/Pub/Get_More_Data_Plus_de_donnees"
-            "/Station%20Inventory%20EN.csv.gz"
+            "https://github.com/earthobservations/testdata/raw/main/ftp.tor.ec.gc.ca/Pub/"
+            "Get_More_Data_Plus_de_donnees/Station%20Inventory%20EN.csv.gz"
         )
 
         payload = None
 
-        # Try original source.
-        session = FTPSession()
+        fs = WholeFileCacheFileSystem(
+            fs=HTTPFileSystem(),
+            cache_storage=cache_dir,
+            expiry_time=CacheExpiry.METAINDEX.value,
+        )
+
         try:
-            response = session.retr(ftp_url)
-            payload = response.content
+            payload = fs.cat(gdrive_url)
         except Exception:
-            log.exception(f"Unable to access FTP server at {ftp_url}")
+            log.exception(f"Unable to access Google drive server at {gdrive_url}")
 
             # Fall back to different source.
             try:
-                response = requests.get(http_url)
-                response.raise_for_status()
-                with gzip.open(BytesIO(response.content), mode="rb") as f:
+                response = fs.cat(http_url)
+                with gzip.open(BytesIO(response), mode="rb") as f:
                     payload = f.read()
             except Exception:
                 log.exception(f"Unable to access HTTP server at {http_url}")
