@@ -1,6 +1,10 @@
-from io import StringIO, BytesIO
-import requests
+# -*- coding: utf-8 -*-
+# Copyright (c) 2018-2021, earthobservations developers.
+# Distributed under the MIT License. See LICENSE for more info.
+from io import BytesIO, StringIO
+
 import PyPDF2
+import requests
 
 
 def read_pdf(url):
